const { getResult } = require('../kv')
const { getValue, setValue } = require('../redis')
const { getResult: mysqlGetResult } = require('../mysql')
const { app: appConfig } = require('../config')
const logger = require('../sgs_loggger')
const { ResponseError } = require('../error')
const { parseQuery, decrypt } = require('../util')

const validType = Array.from(new Array(10), (val, index) => index - 1)

function parseSgsUerAgent (userAgent) {
  if (!userAgent) {
    return { 'type': -1,
      'is_myapp': false,
      'os': 'unknown',
      'is_mobile': true,
      'is_webchat': false,
      'is_mqb': false,
      'valid_type': validType }
  } else {
    userAgent = userAgent.toLowerCase()

    let result = {}

    let products = [
      'sogousearch', 'weixinheadline', 'sogouqudu', 'sogouhealthbook', 'ttnews', 'sogoumobilebrowser',
      'TopTenNews', 'TodayRead', 'XiangYang'
    ]

    let from = userAgent.match(new RegExp(products.join('|')))

    if (!from) {
      result['is_myapp'] = false
      result['type'] = -1
    } else {
      result['is_myapp'] = true
      result['type'] = products.indexOf(from[0])
    }

    result['is_mobile'] = true
    if (userAgent.match(/ios|iphone|ipad|ipod/)) {
      result['os'] = 'ios'
    } else if (userAgent.match(/android/)) {
      result['os'] = 'android'
    } else {
      result['os'] = 'unknown'
      result['is_mobile'] = false
    }

    result['is_webchat'] = userAgent.indexOf('micromessenger') != -1
    result['is_mqb'] = userAgent.indexOf('mqqbrowser') != -1

    result['valid_type'] = validType

    return result
  }
}

function parseRedis (data) {
  if (!data) { return null }

  try {
    let feedDetail = JSON.parse(data)['feed_detail']

    if (feedDetail) {
      let result = {}

      result['openid'] = feedDetail['openid'] || ''
      result['title'] = feedDetail['title'] || ''
      result['nick_name'] = feedDetail['nick_name'] || ''
      result['source_type'] = feedDetail['source_type'] || ''
      result['content'] = feedDetail['content'] || ''
      result['send_time'] = feedDetail['send_time'] || ''

      return result
    } else {
      return null
    }
  } catch (e) {
    logger.error(e)
    return null
  }
}

function parseAd (data) {
  if (!data) { return null }

  try {
    return JSON.parse(data)
  } catch (e) {
    logger.error(e)
    return null
  }
}

function parseKV (data) {
  if (!data) { return null }

  try {
    let articleData = JSON.parse(data)['articleData'] || {}
    let author = articleData['author'] || {}

    if (articleData) {
      let result = {}

      result['openid'] = author['openid'] || ''
      result['title'] = articleData['title'] || ''
      result['nick_name'] = author['nick_name'] || ''
      result['source_type'] = articleData['source_type'] || ''
      result['content'] = articleData['content'] || ''
      result['send_time'] = articleData['publish_time'] || ''

      return result
    } else {
      return null
    }
  } catch (e) {
    logger.error(e)
    return null
  }
}

function buildSgsParams (ctx, next) {
  ctx.xid = appConfig.xid
  ctx.mid = appConfig.mid

  let req = ctx.queryParams['req']
  let userType = ctx.user_type || ctx.queryParams['user_type']

  if (req) {
    let result = JSON.parse(decrypt(req))
    ctx['doc_url'] = result.url || ''
  } else {
    //
    throw new ResponseError({ code: 500, message: 'req param error' })
  }

  ctx.app_type = ctx.queryParams['mat'] || -1

  ctx.sgsUserAgent = parseSgsUerAgent(ctx.request.header['user-agent'])

  if (ctx.sgsUserAgent['is_myapp']) { ctx.app_type = ctx.sgsUserAgent['type'] } else if (ctx.sgsUserAgent['valid_type'].indexOf(ctx.app_type) == -1) {
    ctx.app_type = -1
  }

  ctx['redis_key'] = req + '&appType=' + ctx.app_type

  let referer = ctx.header['referer']

  if (referer) {
    let queryParams = parseQuery(referer)
    ctx.pid = queryParams.get('pid', 0) || queryParams.get('bid', 0) || ''
  } else {
    ctx.pid = ''
  }

  ctx.subid = userType == 1 ? 0 : -1
  ctx.product_type = userType == 1 ? '-3' : (userType == 'oem_wnl'
    ? '-2' : (userType == 'oem_opera' ? '-5' : ''))
  ctx.os = ctx.sgsUserAgent['os']

  let adParams = []
  ctx.adParams = adParams

  adParams.push(['appinfo', encodeURIComponent(ctx.queryParams['appinfo'] || '')])
  adParams.push(['xid', encodeURIComponent(ctx.xid)])
  adParams.push(['mid', encodeURIComponent(ctx.mid)])
  adParams.push(['os', ctx.sgsUserAgent['os']])
  adParams.push(['subid', ctx.subid])
  adParams.push(['ip', encodeURIComponent(ctx.realip || '')])
  adParams.push(['title', ''])
  adParams.push(['docid', encodeURIComponent(ctx.doc_url || '')])
  adParams.push(['type', ctx.product_type])
  adParams.push(['count', 1])
}

async function control (ctx, next) {
  if (!ctx.queryParams || Object.keys(ctx.queryParams).length === 0) {
    return await next()
  }

  let start = new Date().getTime()

  buildSgsParams(ctx, next)

  let userType = ctx.user_type || ctx.queryParams['user_type']

  let adResult = null
  let adPromise = null

  let pid = ctx['pid'] || ctx['bid'] || ''

  let queryString = ctx.adParams.map((v) => {
    return v.join('=')
  }).join('&')

  if (userType == '1' && pid) {
    adPromise = mysqlGetResult(pid, appConfig.adQueryStatement).then(
      (data) => {
        if (!data || !data.length) {
          return null
        } else {
          let firstChannel = data[0]['first_channel']

          if (['无线-银河', '无线其他搜索', '无线输入法信息推荐', '无线其他'].indexOf(firstChannel) != -1) {
            return getResult(appConfig.top_article_advertise_path + '?' + queryString, appConfig.retryTimes)
          } else {
            return null
          }
        }
      }
    )
  } else {
    if (!pid || (['oem_wnl', 'oem_opera'].indexOf(userType) != -1)) {
      adPromise = getResult(appConfig.top_article_advertise_path + '?' + queryString, appConfig.retryTimes)
    }
  }

  let result = await getValue(ctx['redis_key'])
    .then(async (data) => {
      let result = parseRedis(data)

      if (result) {
        return result
      } else {
        result = await getResult(appConfig.feed_detail_path + ('?doc_id=' + (ctx['doc_url'] || '')), appConfig.retryTimes)

        result = parseKV(result)

        if (result) {
          setTimeout(() => {
            setValue(ctx['redis_key'], JSON.stringify({ 'feed_detail': result }))
          }, 0)
        }

        return result
      }
    })

  ctx.result = result

  if (adPromise) {
    adResult = await adPromise
    adResult = parseAd(adResult)
  } else {
    adResult = null
  }

  ctx.adResult = adResult

  logger.debug((new Date().getTime() - start))

  /* 考虑是尾递归让出栈空间
  process.nextTick(()=>{
      next()
  }) */

  await next()
}

module.exports = {
  control
}
