const Koa = require('koa')
const koaBody = require('koa-body')
const koaLogger = require('koa-logger')
const koaFavicon = require('koa-favicon')
const koaUseragent = require('koa')
const logger = require('./utils/logger.js')
const assertMiddleware = require('./middleware/assertMiddleware.js')
const ssrMiddleware = require('./middleware/ssrMiddleware.js')
const exceptionMiddleware = require('./middleware/exceptionMiddleware.js')
const mockOutputMiddleware = require('./middleware/mockOutputMiddleware.js')
const apiMiddleware = require('./middleware/apiMiddleware.js')
const mockInputMiddleware = require('./middleware/mockInputMiddleware.js')
const apiProxyMiddleware = require('./middleware/apiProxyMiddleware.js')
const mergeOptionsMiddleware = require('./middleware/mergeOptionsMiddleware.js')
const uuidMiddleware = require('./middleware/uuidMiddleware.js')
const devHotMiddleware = require('./middleware/devHotMiddleware.js')
const actionsMiddleware = require('./middleware/actionsMiddleware.js')

const path = require('path')
const config = require('config')
const { static: staticPath } = config.get('path')
const { host, port, isDevelopment } = config.get('server')

const url = `${host}${port ? ':' + port : ''}`

const mockConfig = config.get('mock')

const { printRoute } = require('./../webpack/router.js')

function _toString (o) {
  return Object.prototype.toString.call(o)
}

function isAsyncFunction (o) {
  return _toString(o) === '[object AsyncFunction]'
}

function isFunction (o) {
  return _toString(o) === '[object Function]'
}

function isArray (o) {
  return Array.isArray(o)
}

module.exports = function (config, customMiddlewares = {}) {
  const app = new Koa()

  const checkMiddleware = (function (app, customMiddlewares, config) {
    return function (name) {
      let middlewares = customMiddlewares[name]
      if (isFunction(middlewares)) {
        app.use(middlewares(config, app))
      } else if (isAsyncFunction(middlewares)) {
        app.use(middlewares)
      } else if (isArray(middlewares)) {
        middlewares.forEach(middleware => {
          app.use(middleware(config, app))
        })
      } else {
        console.log(`[Hook ${name}]: No Hooks`)
      }
    }
  })(app, customMiddlewares, config)

  // 统一错误捕获与过滤
  app.use(exceptionMiddleware())
  // 入参 mock
  checkMiddleware('mock')
  app.use(mockInputMiddleware())

  app.use(mergeOptionsMiddleware())
  app.use(uuidMiddleware())

  app.use(koaLogger())
  app.use(koaFavicon(path.resolve(staticPath, 'favicon.ico')))
  app.use(koaBody({ patchKoa: true }))

  // 出参 mock
  app.use(mockOutputMiddleware(mockConfig, app))

  config.apiConfig && app.use(apiProxyMiddleware(config, app))
  // api
  app.use(apiMiddleware())
  // actions
  app.use(actionsMiddleware(config.actionsConfig, app))
  // router
  checkMiddleware('router')
  // app.use(ssrMiddleware())
  devHotMiddleware(app)

  app.listen(port, () => {
    logger.info(`✨ 服务已启动 http://${url}\n`)

    printRoute()
  })
}
