const config = require('config')
const pathConfig = config.get('path')
const projectConfig = config.get('project')

const {
  pathConfig: { template: templatePath }
} = require('../config')
const fs = require('fs')
const fse = require('fs-extra')
const path = require('path')
const webpack = require('webpack')
const clientConfig = require('./../../webpack/dev/webpack.client.config')
const serverConfig = require('./../../webpack/dev/webpack.server.config')
const webpackDevMiddleware = require('webpack-dev-middleware')
const { devMiddleware, hotMiddleware } = require('koa-webpack-middleware')
const SingleEntryPlugin = require('webpack/lib/SingleEntryPlugin')
const MultiEntryPlugin = require('webpack/lib/MultiEntryPlugin')
const { createBundleRenderer } = require('vue-server-renderer')
const koaSend = require('koa-send')

const { publicPath } = require('./../../webpack/output.js')

/* --- 额外 Script 资源 --- */
const extraScriptNames = config.get('extraScript') || []
const createHeaderScriptPath = require('../utils/createHeaderScriptPath')
const headerScriptPath = createHeaderScriptPath(extraScriptNames)
/* ----- */
// entry
const webpackClientEntry = JSON.parse(JSON.stringify(clientConfig.entry))
const webpackServerEntry = JSON.parse(JSON.stringify(serverConfig.entry))

module.exports = function(app, options) {
  const dllJSON = require(path.resolve(pathConfig.dll, 'index.json'))
  const dllVendor = Object.values(dllJSON)
    .reduce((prev, cur) => {
      let val = Object.values(cur)
      prev = prev.concat(val)
      return prev
    }, [])
    .map(v => v.substring(1))

  const readFile = (fs, file) => {
    try {
      return fs.readFileSync(path.join(clientConfig.output.path, file), 'utf-8')
    } catch (e) {}
  }

  clientConfig.entry = {
    global: [
      'webpack-hot-middleware/client?path=/__webpack_hmr&noInfo=false&reload=true',
      webpackClientEntry['global']
    ]
  }

  serverConfig.entry = {
    global: webpackServerEntry['global']
  }

  const template = fs.readFileSync(
    path.resolve(templatePath, 'ssr/template.html'),
    'utf-8'
  )

  const clientCompiler = webpack(clientConfig)
  const clientDevInstance = webpackDevMiddleware(clientCompiler, {
    publicPath: clientConfig.output.publicPath,
    stats: clientConfig.stats
  })
  let clientPromise = function() {
    return new Promise((resolve, reject) => {
      clientDevInstance.waitUntilValid(function(stats) {
        stats = stats.toJson()
        stats.errors.forEach(err => console.error(err))
        stats.warnings.forEach(err => console.warn(err))
        if (stats.errors.length) {
          reject(stats)
        } else {
          let clientManifest = JSON.parse(
            readFile(
              clientDevInstance.fileSystem,
              'vue-ssr-client-manifest.json'
            )
          )
          resolve(clientManifest)
        }
      })
    })
  }

  const serverCompiler = webpack(serverConfig)
  const serverDevInstance = webpackDevMiddleware(serverCompiler, {
    publicPath: '/',
    stats: serverConfig.stats
  })
  let serverPromise = function() {
    return new Promise((resolve, reject) => {
      serverDevInstance.waitUntilValid(function(stats) {
        stats = stats.toJson()
        stats.errors.forEach(err => console.error(err))
        stats.warnings.forEach(err => console.warn(err))
        if (stats.errors.length) {
          reject(stats)
        } else {
          let bundle = JSON.parse(
            readFile(serverDevInstance.fileSystem, 'vue-ssr-server-bundle.json')
          )
          resolve(bundle)
        }
      })
    })
  }

  /* 传入重置后 clientManifest & bundle */
  function renderer(clientManifest, bundle, ctx) {
    const context = {
      url: ctx.originalUrl,
      traceScriptPath: '',
      headerScriptPath: headerScriptPath,
      title: ctx.__SSR_TITLE__ || projectConfig.title,
      __SSR_CONTENT__: ctx.__SSR_CONTENT__
    }

    return new Promise((res, rej) => {
      createBundleRenderer(bundle, {
        template,
        clientManifest,
        runInNewContext: false
      }).renderToString(context, (err, html) => {
        if (err) {
          console.log(err)
          console.log(err.stack)
          rej(err)
        } else {
          res(html)
        }
      })
    })
  }

  app.use(async (ctx, next) => {
    let reqPath = ctx.path
    if (!reqPath.startsWith(`/${projectConfig.name}/${projectConfig.html}`)) {
      await next()
    } else {
      try {
        // #issure folder is path of entry
        let [, , , folder, router] = reqPath.split('/')

        clientCompiler.apply(
          new SingleEntryPlugin(
            pathConfig.root,
            webpackClientEntry[folder],
            folder
          )
        )
        clientDevInstance.invalidate()

        serverCompiler.apply(
          new SingleEntryPlugin(
            pathConfig.root,
            webpackServerEntry[folder],
            folder
          )
        )
        serverDevInstance.invalidate()
        let [clientManifest, bundle] = await Promise.all([
          clientPromise(),
          serverPromise()
        ])
        // 重新修订 dllVendor 的 path
        let realDllVendor = dllVendor.map(v => {
          return v.substring(publicPath.length - 1)
        })
        clientManifest.async = []
        let initial = realDllVendor.concat([
          'js/global.js',
          `js/${folder}.js`,
          `js/${router}.js`
        ])
        Object.assign(clientManifest, { initial })
        Object.assign(bundle, { entry: `js/${folder}-server.js` })
        let html = await renderer(clientManifest, bundle, ctx)

        console.clear()
        console.log(html)

        ctx.body = html
      } catch (e) {
        console.error(e.stack)
        ctx.body = e
      }
    }
  })

  app.use(async (ctx, next) => {
    ctx.status = 200
    await clientDevInstance(ctx.req, ctx.res, async () => {
      await next()
    })
  })

  app.use(async (ctx, next) => {
    let reqPath = ctx.path
    if (reqPath === '/__webpack_hmr') {
      await next()
    } else {
      // publicPath -> ssr
      // static -> dllWebpack
      let prefix = publicPath
      let maxage = 365 * 24 * 60 * 60 * 1000
      // del dist prefix
      let realReqPath = reqPath.substring(prefix.length - 1)
      const exists = await fse.pathExists(`${pathConfig.static}${realReqPath}`)
      let result
      if (exists) {
        result = await koaSend(ctx, realReqPath, {
          root: pathConfig.static,
          maxage
        })
      }

      if (!result) {
        await next()
      }
    }
  })

  app.use(hotMiddleware(clientCompiler))
}
