const {
  pathConfig: { template: templatePath }
} = require('./../config')

const config = require('config')
const pathConfig = config.get('path')
const projectConfig = config.get('project')
const path = require('path')
const fs = require('fs')
const LRU = require('lru-cache')
const { createBundleRenderer } = require('vue-server-renderer')
const { serverEntry } = require('./../../webpack/entry.js')
const { publicPath } = require('./../../webpack/output.js')
const { NODE_ENV } = process.env
const InvalidRouteException = require('../errors/exceptions/invalid-route.exception')
const glob = require('glob')

/* --- 额外 Script 资源 --- */
const extraScriptNames = config.get('extraScript') || []
const createHeaderScriptPath = require('../utils/createHeaderScriptPath')
const headerScriptPath = createHeaderScriptPath(extraScriptNames)
/* ----- */

const middleware = ({ needInit }, app) => {
  // template
  const template = fs.readFileSync(
    path.resolve(templatePath, 'ssr/template.html'),
    'utf-8'
  )
  let bundle = require(path.resolve(
    pathConfig.dist,
    'vue-ssr-server-bundle.json'
  ))

  let clientManifestOrigin = require(path.resolve(
    pathConfig.dist,
    'vue-ssr-client-manifest.json'
  ))

  const dllJSON = require(path.resolve(pathConfig.dll, 'index.json'))
  const dllVendor = Object.values(dllJSON)
    .reduce((prev, cur) => {
      let val = Object.values(cur)
      prev = prev.concat(val)
      return prev
    }, [])
    .map(v => v.substring(1))

  /* --- 预热 ---- */
  function createBundleRendererInstance(folder, router) {
    let clientManifest = JSON.parse(JSON.stringify(clientManifestOrigin))
    let oldInitial = clientManifest.initial
    let oldAsync = clientManifest.async
    // 入口
    let entry = `js/${folder}-server.js`
    /* 重置 initial & entry */
    let initial = oldInitial.filter(v => {
      return ['js/global', 'css/global', `js/${folder}`, `css/${folder}`].some(
        k => {
          return v.startsWith(k)
        }
      )
    })

    // 重置 global 在上面
    initial = initial.reduce((prev, cur) => {
      if (
        ['js/global', 'css/global'].some(k => {
          return cur.startsWith(k)
        })
      ) {
        prev.unshift(cur)
      } else {
        prev.push(cur)
      }
      return prev
    }, [])

    let realDllVendor = dllVendor.map(v => {
      return v.substring(publicPath.length - 1)
    })

    let async = oldAsync.filter(v => {
      let [, filepath] = v.split('/')
      let filename = filepath.split('.').shift()
      return filename === router
    })

    clientManifest.async = async
    initial = realDllVendor.concat(initial)
    Object.assign(clientManifest, { initial })
    Object.assign(bundle, { entry })

    let bundleRenderer = createBundleRenderer(bundle, {
      template,
      clientManifest,
      basedir: pathConfig.dist,
      runInNewContext: false
    })

    return bundleRenderer
  }

  let bundleRendererInstanceMapping = Object.keys(serverEntry).reduce(
    (prev, folder) => {
      // 获取当前 folder 下的文件
      let folderRouteBundleRendererInstance = glob
        .sync(path.resolve(pathConfig.pages, `./${folder}/views/*.vue`), {
          noext: 'csr.vue'
        })
        .filter(v => {
          let filename = path.basename(v)
          return !filename.includes('.csr.vue')
        })
        .reduce((t, c) => {
          let filename = path.basename(c, '.vue')
          t[filename] = createBundleRendererInstance(folder, c)

          return t
        }, {})

      prev[folder] = folderRouteBundleRendererInstance
      return prev
    },
    {}
  )

  Object.values(needInit).forEach(params => {
    let { url } = params
    let [, , , folder, route] = url.split('/')
    bundleRendererInstanceMapping[folder][route].renderToString(
      Object.assign(params, {
        headerScriptPath,
        traceScriptPath: `<script src="https://dlweb.sogoucdn.com/hhytrace/trace_${formatDate()}.js"></script>`
      })
    )
  })

  /* --- End ---- */

  const handleError = (err, ctx) => {
    if (err.url) {
      ctx.redirect(err.url)
    } else if (err.code === 404) {
      ctx.status = 404
      ctx.body = '404 | Page Not Found'
    } else {
      // Render Error Page or Redirect
      ctx.status = 500
      ctx.body = '500 | Internal Server Error'

      console.error(`error during render : ${ctx.path}`)
      console.error(err.stack)
    }
  }

  function formatDate() {
    var d = new Date(),
      t = d.getMonth() + 1,
      n = d.getDate(),
      g = d.getHours()

    return (
      d.getFullYear() +
      (String(t).length < 2 ? '0' + t : t) +
      (String(n).length < 2 ? '0' + n : n) +
      (String(g).length < 2 ? '0' + g : g)
    )
  }

  return async (ctx, next) => {
    let reqPath = ctx.path

    if (!reqPath.startsWith(`/${projectConfig.name}/${projectConfig.html}`)) {
      return next()
    }

    // #issure folder is path of entry
    try {
      // logger
      let [, , , folder, route] = reqPath.split('/')
      let bundleRendererInstance = bundleRendererInstanceMapping[folder][route]
      if (!bundleRendererInstance) {
        throw new Error('404 | Entry Not Found')
      }

      ctx.Monitor.benchmark('start', 'SSR Renderer')
      const context = {
        url: ctx.originalUrl,
        headerScriptPath: headerScriptPath,
        traceScriptPath: `<script src="https://dlweb.sogoucdn.com/hhytrace/trace_${formatDate()}.js"></script>`,
        title: ctx.__SSR_TITLE__ || projectConfig.title,
        __SSR_CONTENT__: ctx.__SSR_CONTENT__
      }

      const renderer = function() {
        return new Promise((res, rej) => {
          bundleRendererInstance.renderToString(context, (err, html) => {
            if (err) {
              console.log(err)
              console.log(err.stack)
              rej(err)
            } else {
              res(html)
            }
          })
        })
      }

      ctx.Monitor.benchmark('start', 'RenderToString')
      let html = await renderer()
      ctx.Monitor.benchmark('end', 'RenderToString')

      ctx.body = html
      ctx.Monitor.benchmark('end', 'SSR Renderer')
    } catch (e) {
      let err = new InvalidRouteException(e.stack)
      // handleError(e, ctx)
      throw err
    }
  }
}

module.exports = middleware
